from netgen.csg import *
from ngsolve import *


def MakeGeometry():
    geometry = CSGeometry()

    
    r_noyau = 0.015
    r_bobine_int=0.016
    r_bobine_ext=0.020
    box_ext = 0.05
    h = 0.03
    
    box = OrthoBrick(Pnt(-box_ext,-box_ext,-box_ext),Pnt(box_ext,box_ext,box_ext)).bc("outer")

    core = Cylinder(Pnt(0,0,0), Pnt(0,0,1), r_noyau)*\
           OrthoBrick(Pnt(-1,-1,-h/2),Pnt(1,1,h/2)).maxh(0.2).mat("core")
    
    coil = (Cylinder(Pnt(0,0,0), Pnt(0,0,1), r_bobine_ext) - \
            Cylinder(Pnt(0,0,0), Pnt(0,0,1), r_bobine_int)) * \
            OrthoBrick(Pnt(-1,-1,-h/2),Pnt(1,1,h/2)).maxh(0.2).mat("coil")
    
    geometry.Add ((box-core-coil).mat("air"))
    geometry.Add (core)
    geometry.Add (coil)
    return geometry


ngmesh = MakeGeometry().GenerateMesh(maxh=0.01)
ngmesh.Save("coil.vol")
mesh = Mesh(ngmesh)

# curve elements for geometry approximation
mesh.Curve(5)

ngsglobals.msg_level = 5

fes = HCurl(mesh, order=3, dirichlet="outer", nograds = True)

# u and v refer to trial and test-functions in the definition of forms below
u = fes.TrialFunction()
v = fes.TestFunction()

mur = mesh.MaterialCF({ "core" : 1000 }, default=1)
mu0 = 1.257e-6
nu = 1/(mu0*mur)

a = BilinearForm(fes)#, symmetric=True)
a += nu*curl(u)*curl(v)*dx + 1e-6*nu*u*v*dx

c = Preconditioner(a, type="bddc")
# c = Preconditioner(a, type="multigrid", flags = { "smoother" : "block" } )

f = LinearForm(fes)

#vecteur=CoefficientFunction((-cos(atan(y/x)),sin(atan(y/x)),0))* dx("coil")
#courant= vecteur* v 
#f += CoefficientFunction((-cos(atan(y/x)),sin(atan(y/x)),0)) * v * dx("coil")



vecteur=CoefficientFunction((sin(atan(y/x)),-cos(atan(y/x)),0))*\
         CoefficientFunction([1 if mat=="coil" else 0 for mat in mesh.GetMaterials()])

f += CoefficientFunction((sin(atan(y/x)),-cos(atan(y/x)),0))* v * dx("coil")

u = GridFunction(fes)


with TaskManager():
    a.Assemble()
    f.Assemble()
    solver = CGSolver(mat=a.mat, pre=c.mat)
    u.vec.data = solver * f.vec
#u.Deriv()

Draw (u.Deriv(), mesh, "B-fieldCCL", draw_surf=False)

